/*
    File: SimpleFormat.r

    Copyright (c) 1993-6, Adobe Systems Incorporated.
    All rights reserved.

	Rez file for SimpleFormat file format example module.
*/

/********************************************************************************/
/* Defines required by included files */

#define plugInName			"SimpleFormat"
#define VersionString		"4.0"

/********************************************************************************/

#if Macintosh
#include "Types.r"
#include "Systypes.r"
#include "PIGeneral.r"
#include "PIUtilities.r"
#include "DialogUtilities.r"
#endif

#if MSWindows
#include "PIGeneral.h"
#include "PIUtilities.r"
#include "WinDialogUtils.r"
#endif

#include "PIActions.h"

/*****************************************************************************/

#define plugInComment 		"simpleformat example file format plug-in"
#define vendorName			"AdobeSDK"

#define	ourSuiteID			'sdK4'
#define ourClassID			'simP'
#define ourEventID			typeNull // must be this

#define keyFoo				'fooB'
#define keyBar				'barF'

/*****************************************************************************/

resource 'PiPL' (ResourceID, purgeable)
	{
		{
		Kind { ImageFormat },
		Name { plugInName },
		Version { (latestFormatVersion << 16) | latestFormatSubVersion },

		#if Macintosh
		Code68K { ImageFormat, $$ID },
		CodePowerPC { 0, 0, "" },
		#endif

		#if MSWindows
		CodeWin32X86 { "ENTRYPOINT" },
		#endif
	
		HasTerminology { ourClassID, ourEventID, ResourceID, "" },
		/* class ID, event ID, aete ID, uniqueString */
		
		SupportedModes
			{
			doesSupportBitmap, doesSupportGrayScale,
			doesSupportIndexedColor, doesSupportRGBColor,
			doesSupportCMYKColor, doesSupportHSLColor,
			doesSupportHSBColor, doesSupportMultichannel,
			doesSupportDuotone, doesSupportLABColor
			},
			
		EnableInfo { "true" },
	
		FmtFileType { '8B1F', '8BIM' },
		ReadTypes { { '8B1F', '    ' } },
		ReadExtensions { { 'SME ' } },
		WriteExtensions { { 'SME ' } }, 
		FormatFlags { fmtSavesImageResources, fmtCanRead, fmtCanWrite, fmtCanWriteIfRead },
		/* canRead, canWrite, canWriteIfRead, savesResources */
		FormatMaxSize { { 32767, 32767 } },
		FormatMaxChannels { {   1, 24, 24, 24, 24, 24, 
							   24, 24, 24, 24, 24, 24 } }
		}
	};


resource 'PiMI' (ResourceID, purgeable)
{
	latestFormatVersion, 	/* Version, subVersion, and priority of the interface */
	latestFormatSubVersion,
	0,
	supportsBitMap +
	supportsGrayScale +
	supportsIndexedColor +
	supportsRGBColor +
	supportsCMYKColor +
	supportsHSLColor +
	supportsHSBColor +
	supportsMultichannel +
	supportsDuotone +
	supportsLABColor,			/* Supported Image Modes */
	'    ',						/* Required host */
	
	{
		canRead,
		cannotReadAll,
		canWrite,
		canWriteIfRead,
		savesResources,
		{  1, 16, 16, 16,		/* Maximum # of channels for each plug-in mode */
		  16, 16, 16, 16,
		  16, 16,  0,  0,
		   0,  0,  0,  0 },
		32767,				/* Maximum rows allowed in document */
		32767,				/* Maximum columns allowed in document */
		'8B1F',				/* The file type if we create a file. */
		'8BIM',				/* The creator type if we create a file. */
		{					/* The type-creator pairs supported. */
			'8B1F', '    '
		},
		{					/* The extensions supported. */
		}
	},
	
};

/* About String resource */

resource StringResource (AboutID, "About Text", purgeable)
{
	plugInName "\n\n"
	"Version " VersionString " "
	"Release " ReleaseString "\n"
	"Copyright  1992-6 Adobe Systems Incorporated.\n"
	"All rights reserved.\n\n"
	"A plug-in image format module that reads and writes "
	"Simple Format files in Adobe Photoshop."
};

/* scripting dictionary resource */

resource 'aete' (ResourceID, purgeable)
{
	1, 0, english, roman,									/* aete version and language specifiers */
	{
		vendorName,											/* vendor suite name */
		"Adobe example plug-ins",							/* optional description */
		ourSuiteID,											/* suite ID */
		1,													/* suite code, must be 1 */
		1,													/* suite level, must be 1 */
		{},													/* structure for filters */
		{													/* non-filter plug-in class here */
			vendorName " simpleFormat",						/* unique class name */
			ourClassID,										/* class ID, must be unique or Suite ID */
			plugInName " plug-in",							/* optional description */
			{												/* define inheritance */
				"<Inheritance>",							/* must be exactly this */
				keyInherits,								/* must be keyInherits */
				classFormat,								/* parent: Format, Import, Export */
				"parent class format",						/* optional description */
				flagsSingleProperty,						/* if properties, list below */
							
				"foo",
				keyFoo,
				typeBoolean,
				"foobar",
				flagsSingleProperty,
				
				"bar",
				keyBar,
				typeBoolean,
				"foobar",
				flagsSingleProperty
				/* no properties */
			},
			{}, /* elements (not supported) */
			/* class descriptions */
		},
		{}, /* comparison ops (not supported) */
		{}	/* any enumerations */
	}
};